source("read_mar.r")
# マルコフ基底 
# 5x5cont.mat.mar は 5x5 分割表のマルコフ基底で 
# 4ti2 のコマンド markov の出力結果とする
markov_b <- read_mar("5x5cont.mar")

# markov_b を -1 倍したものを付け加える
tm <- t(markov_b)
tmm <- -tm
markov_b <- matrix(append(tm, tmm), nrow(tm))
markov_b <- t(markov_b)

# p(y + z)/p(y) の計算 (z はマルコフ基底のいずれか)
# p(y) = 1/Z 1/(y[1]! ... y[length(y)]!)
prob_r <- function(y, z) {
	len <- length(y)
	r <- 1
	# マルコフ基底の成分が 1, 0, -1 の場合で場合分けして計算
	# マルコフ基底の成分で 1, 0, -1 以外が出る場合は使えないことに注意
	for (i in 1:len) {
		if (z[i] == 1) {
			r <- r / (y[i] + 1)
		} else if (z[i] == -1) {
			r <- r * y[i]
		}
	}
	return(r)
}

# カイ２乗統計量の計算
my_chisq <- function(x, fitted_v) {
	n <- length(x)
	sum <- 0
	for (i in 1:n) {
		sum <- sum + (x[i] - fitted_v[i])^2/fitted_v[i]
	}
	return(sum)
}

test_my_chisq <- function() {
	return(my_chisq(state, fv))
}

# 5x5 分割表 mcmc 
# 引数
# n      : 取り出す数
# burnin : 最初からいくら捨てるか
# state  : 最初の状態 
c5x5mcmc <- function(n = 10000, burnin = 10000, state = c(2,1,1,0,0,8,3,3,0,0,0,2,1,1,1,0,0,0,1,1,0,0,0,0,1)) {
	ctest <- chisq.test(t(matrix(state, nrow=5)))
	chi <- ctest$statistic  # 最初の状態のカイ2乗統計量
	m <- ctest$expected     # 当てはめ値
	fv <- as.vector(t(m))
	print(chi)
	print(fv)

	n_mar <- nrow(markov_b)
	select <- sample(c(1:n_mar), n, replace=TRUE) # マルコフ基底 markov_b のどれを選ぶか
	rr <- runif(n)
	for (i in 1:burnin) {
		index <- select[i]
		r <- prob_r(state, markov_b[index, ])	
		if (r > rr[i]) {
			state <- state + markov_b[index,]
		}
	}

	count <- 0
	select <- sample(c(1:n_mar), n, replace=TRUE) # マルコフ基底 markov_b のどれを選ぶか
	rr <- runif(n)
	for (i in 1:n) {
		index <- select[i]
		r <- prob_r(state, markov_b[index, ])	
		if (r > rr[i]) {
			state <- state + markov_b[index,]
		}
		tchi <- my_chisq(state, fv)
		if (tchi > chi) {
			count <- count + 1
			print("*")
		}
		print(state) # サンプルの表示
	}
	cat("p 値の近似値\n")
	print(count/n)
}

cat("c5x5mcmc(n, burnin, 初期値) で実行\n")
cat("e.g. c5x5mcmc(10000, 10000, c(2,1,1,0,0,8,3,3,0,0,0,2,1,1,1,0,0,0,1,1,0,0,0,0,1))\n")

