source("read_mar.r")

# p(y + z)/p(y) の計算 (z はマルコフ基底のいずれか)
# p(y) = 1/Z 1/(y[1]! ... y[length(y)]!)
prob_r <- function(y, z) {
	len <- length(y)
	r <- 1
	for (i in 1:len) {
		if (z[i] == 1) {
			r <- r / (y[i] + 1)
		} else if (z[i] == -1) {
			r <- r * y[i]
		}
	}
	return(r)
}

# カイ２乗統計量の計算
my_chisq <- function(x, fitted_v) {
	n <- length(x)
	sum <- 0
	for (i in 1:n) {
		sum <- sum + (x[i] - fitted_v[i])^2/fitted_v[i]
	}
	return(sum)
}

	
test_my_chisq <- function() {
	return(my_chisq(state, fv))
}

# 最初の状態  state
# 5x5 分割表のマルコフ基底で 4ti2 markov の出力結果 mar_filename
# 取り出す数  n
# 最初からいくら捨てるか burnin
# 乱数seed  seed
c5x5_mcmc <- function(state, mar_filename, n=10000, burnin=10000, seed = 0) {
	set.seed(seed)
	markov_b <- read_mar(mar_filename)
	
	# markov_b を -1 倍したものを付け加える
	tm <- t(markov_b)
	tmm <- -tm
	markov_b <- matrix(append(tm, tmm), nrow(tm))
	markov_b <- t(markov_b)
	
	ctest <- chisq.test(t(matrix(state, nrow=5)))
	chi <- ctest$statistic  # 最初の状態のカイ2乗統計量
	m <- ctest$expected     # 当てはめ値
	fv <- as.vector(t(m))
	print(chi)
	print(fv)
	
	n_mar <- nrow(markov_b)
	select <- sample(c(1:n_mar), n, replace=TRUE) # マルコフ基底 markov_b のどれを選ぶか
	rr <- runif(n)
	for (i in 1:burnin) {
		index <- select[i]
		r <- prob_r(state, markov_b[index, ])	
		if (r > rr[i]) {
			state <- state + markov_b[index,]
		}
	}
	
	count <- 0
	select <- sample(c(1:n_mar), n, replace=TRUE) # マルコフ基底 markov_b のどれを選ぶか
	rr <- runif(n)
	for (i in 1:n) {
		index <- select[i]
		r <- prob_r(state, markov_b[index, ])	
		if (r > rr[i]) {
			state <- state + markov_b[index,]
		}
		tchi <- my_chisq(state, fv)
		if (tchi > chi) {
			count <- count + 1
			print("*")
		}
		print(state)
	}
	print("approximate p-value")
	print(count/n)
	return(count/n)
}

test_c5x5_mcmc <- function()
{
state <- c(2,1,1,0,0,8,3,3,0,0,0,2,1,1,1,0,0,0,1,1,0,0,0,0,1)  # 最初の状態
mar_filename <- "5x5cont.mar"
	c5x5_mcmc(state, mar_filename)
}
